"""GSK 适配器消息。"""

from typing import Literal, Optional, Type, Union
from typing_extensions import Self

from iamai.message import Message, MessageSegment

__all__ = ["GSKMessage", "GSKMessageSegment", "escape"]


class GSKMessage(Message["GSKMessageSegment"]):
    """GSK 消息。"""

    @classmethod
    def get_segment_class(cls) -> Type["GSKMessageSegment"]:
        """获取消息字段类。

        Returns:
            消息字段类。
        """
        return GSKMessageSegment


class GSKMessageSegment(MessageSegment["GSKMessage"]):
    """GSK 消息字段。"""

    @classmethod
    def get_message_class(cls) -> Type[GSKMessage]:
        """获取消息类。

        Returns:
            消息类。
        """
        return GSKMessage

    @classmethod
    def from_str(cls, msg: str) -> Self:
        """用于将 `str` 转换为消息字段。

        Args:
            msg: 要解析为消息字段的数据。

        Returns:
            由 `str` 转换的消息字段。
        """
        return cls.text(msg)

    def __str__(self) -> str:
        """返回消息字段的文本表示。

        Returns:
            消息字段的文本表示。
        """
        if self.type == "text":
            return self.data.get("text", "")
        return self.get_cqcode()

    def get_cqcode(self) -> str:
        """获取此消息字段的 CQ 码形式。

        Returns:
            此消息字段的 CQ 码形式。
        """
        if self.type == "text":
            return escape(self.data.get("text", ""), escape_comma=False)

        params = ",".join(
            [f"{k}={escape(str(v))}" for k, v in self.data.items() if v is not None]
        )
        return f'[CQ:{self.type}{"," if params else ""}{params}]'

    @classmethod
    def text(cls, text: str) -> Self:
        """纯文本"""
        return cls(type="text", data={"text": text})

    @classmethod
    def face(cls, id_: int) -> Self:
        """QQ 表情"""
        return cls(type="face", data={"id": str(id_)})

    @classmethod
    def image(
        cls,
        file: str,
        type_: Optional[Literal["flash"]] = None,
        cache: bool = True,
        proxy: bool = True,
        timeout: Optional[int] = None,
    ) -> Self:
        """图片"""
        return cls(
            type="image",
            data={
                "file": file,
                "type": type_,
                "cache": cache,
                "proxy": proxy,
                "timeout": timeout,
            },
        )

    @classmethod
    def record(
        cls,
        file: str,
        magic: bool = False,
        cache: bool = True,
        proxy: bool = True,
        timeout: Optional[int] = None,
    ) -> Self:
        """语音"""
        return cls(
            type="record",
            data={
                "file": file,
                "magic": magic,
                "cache": cache,
                "proxy": proxy,
                "timeout": timeout,
            },
        )

    @classmethod
    def video(
        cls,
        file: str,
        cache: bool = True,
        proxy: bool = True,
        timeout: Optional[int] = None,
    ) -> Self:
        """短视频"""
        return cls(
            type="video",
            data={"file": file, "cache": cache, "proxy": proxy, "timeout": timeout},
        )

    @classmethod
    def at(cls, qq: Union[int, Literal["all"]]) -> Self:  # pylint: disable=invalid-name
        """@某人"""
        return cls(type="at", data={"qq": str(qq)})

    @classmethod
    def rps(cls) -> Self:
        """猜拳魔法表情"""
        return cls(type="rps", data={})

    @classmethod
    def dice(cls) -> Self:
        """掷骰子魔法表情"""
        return cls(type="dice", data={})

    @classmethod
    def shake(cls) -> Self:
        """窗口抖动 (戳一戳)"""
        return cls(type="shake", data={})

    @classmethod
    def poke(cls, type_: str, id_: int) -> Self:
        """戳一戳"""
        return cls(type="poke", data={"type": type_, "id": str(id_)})

    @classmethod
    def anonymous(cls, ignore: Optional[bool] = None) -> Self:
        """匿名发消息"""
        return cls(type="anonymous", data={"ignore": ignore})

    @classmethod
    def share(
        cls,
        url: str,
        title: str,
        content: Optional[str] = None,
        image: Optional[str] = None,
    ) -> Self:
        """链接分享"""
        return cls(
            type="share",
            data={"url": url, "title": title, "content": content, "image": image},
        )

    @classmethod
    def contact(cls, type_: Literal["qq", "group"], id_: int) -> Self:
        """推荐好友/推荐群"""
        return cls(type="contact", data={"type": type_, "id": str(id_)})

    @classmethod
    def contact_friend(cls, id_: int) -> Self:
        """推荐好友"""
        return cls(type="contact", data={"type": "qq", "id": str(id_)})

    @classmethod
    def contact_group(cls, id_: int) -> Self:
        """推荐好友"""
        return cls(type="contact", data={"type": "group", "id": str(id_)})

    @classmethod
    def location(
        cls, lat: float, lon: float, title: Optional[str], content: Optional[str] = None
    ) -> Self:
        """位置"""
        return cls(
            type="location",
            data={"lat": str(lat), "lon": str(lon), "title": title, "content": content},
        )

    @classmethod
    def music(cls, type_: Literal["qq", "163", "xm"], id_: int) -> Self:
        """音乐分享"""
        return cls(type="music", data={"type": type_, "id": str(id_)})

    @classmethod
    def music_custom(
        cls,
        url: str,
        audio: str,
        title: str,
        content: Optional[str] = None,
        image: Optional[str] = None,
    ) -> Self:
        """音乐自定义分享"""
        return cls(
            type="music",
            data={
                "type": "custom",
                "url": url,
                "audio": audio,
                "title": title,
                "content": content,
                "image": image,
            },
        )

    @classmethod
    def reply(cls, id_: int) -> Self:
        """回复"""
        return cls(type="reply", data={"id": str(id_)})

    @classmethod
    def node(cls, id_: int) -> Self:
        """合并转发节点"""
        return cls(type="node", data={"id": str(id_)})

    @classmethod
    def node_custom(cls, user_id: int, nickname: str, content: "GSKMessage") -> Self:
        """合并转发自定义节点"""
        return cls(
            type="node",
            data={
                "user_id": str(user_id),
                "nickname": str(nickname),
                "content": content,
            },
        )

    @classmethod
    def xml_message(cls, data: str) -> Self:
        """XML 消息"""
        return cls(type="xml", data={"data": data})

    @classmethod
    def json_message(cls, data: str) -> Self:
        """JSON 消息"""
        return cls(type="json", data={"data": data})


def escape(string: str, *, escape_comma: bool = True) -> str:
    """对 CQ 码中的特殊字符进行转义。

    Args:
        string: 待转义的字符串。
        escape_comma: 是否转义 `,`。

    Returns:
        转义后的字符串。
    """
    string = string.replace("&", "&amp;").replace("[", "&#91;").replace("]", "&#93;")
    if escape_comma:
        string = string.replace(",", "&#44;")
    return string



from iamai import Plugin
from iamai.adapter.apscheduler import scheduler_decorator


@scheduler_decorator(
    trigger="interval", trigger_args={"seconds": 10}, override_rule=True
)
class Schedule(Plugin):
    async def handle(self) -> None:
        print("scheduler_decorator", self.event.type, self.event.adapter)

    async def rule(self) -> bool:
        return False



"""bililive 协议适配器。

本适配器适配了 bililive 协议。
协议详情请参考: [xfgryujk/blivedm](https://github.com/xfgryujk/blivedm) 。

TODO:
    - [x] 扫码登录
    - [x] 本地缓存cookie登录
    - [ ] onebot 适配
    - [ ] api
"""

import os
import re  # noqa: F401
import sys
import json
import time
import zlib
import struct
import asyncio
from math import log  # noqa: F401
from functools import partial
from abc import abstractmethod
from collections import namedtuple
from os.path import join, split, abspath, dirname  # noqa: F401
from typing import TYPE_CHECKING, Any, Dict, NamedTuple  # noqa: F401

import qrcode
import aiohttp
from genericpath import exists
from aiohttp.client import ClientSession

from iamai.utils import DataclassEncoder  # noqa: F401
from iamai.adapter.utils import WebSocketAdapter
from iamai.log import logger, error_or_exception

from .event import *  # noqa: F403
from .message import *  # noqa: F403
from .config import Config
from .event import get_event_class

if TYPE_CHECKING:
    from .message import T_BililiveMSG  # noqa: F401

__all__ = ["BililiveAdapter"]

ROOM_INIT_URL = "https://api.live.bilibili.com/xlive/web-room/v1/index/getInfoByRoom"
DANMAKU_SERVER_CONF_URL = (
    "https://api.live.bilibili.com/xlive/web-room/v1/index/getDanmuInfo"
)
DEFAULT_DANMAKU_SERVER_LIST = [
    {
        "host": "broadcastlv.chat.bilibili.com",
        "port": 2243,
        "wss_port": 443,
        "ws_port": 2244,
    }
]
QRCODE_REQUEST_URL = "http://passport.bilibili.com/qrcode/getLoginUrl"
CHECK_LOGIN_RESULT = "http://passport.bilibili.com/qrcode/getLoginInfo"
SEND_URL = "https://api.live.bilibili.com/msg/send"
MUTE_USER_URL = (
    "https://api.live.bilibili.com/xlive/web-ucenter/v1/banned/AddSilentUser"
)
ROOM_SLIENT_URL = "https://api.live.bilibili.com/xlive/web-room/v1/banned/RoomSilent"
ADD_BADWORD_URL = (
    "https://api.live.bilibili.com/xlive/web-ucenter/v1/banned/AddShieldKeyword"
)
DEL_BADWORD_URL = (
    "https://api.live.bilibili.com/xlive/web-ucenter/v1/banned/DelShieldKeyword"
)
HEADER_STRUCT = struct.Struct(">I2H2I")
HeaderTuple = namedtuple(
    "HeaderTuple", ("pack_len", "raw_header_size", "ver", "operation", "seq_id")
)
WS_BODY_PROTOCOL_VERSION_INFLATE = 0
WS_BODY_PROTOCOL_VERSION_NORMAL = 1
WS_BODY_PROTOCOL_VERSION_DEFLATE = 2

user_cookies = aiohttp.cookiejar.CookieJar()


class BililiveAdapter(WebSocketAdapter[BililiveEvent, Config]):  # noqa: F405
    """bililive 协议适配器。"""

    name: str = "bililive"
    Config = Config
    _gateway_response = {}  # type: ignore
    _host_server_list = DEFAULT_DANMAKU_SERVER_LIST
    _api_response: Dict[Any, Any]
    _api_response_cond: asyncio.Condition = None  # type: ignore
    _api_id: int = 0
    _heartbeat_interval = 30
    _retry_count = 0

    def __getattr__(self, item):  # type: ignore
        return partial(self.call_api, item)

    async def startup(self):
        """初始化适配器。"""
        self.adapter_type = self.config.adapter_type  # type: ignore
        if self.adapter_type == "websocket":  # type: ignore
            self.adapter_type = "ws"  # type: ignore
        self.reconnect_interval = self.config.reconnect_interval  # type: ignore
        self.room_id = self.config.room_id  # type: ignore
        self.session_data_path = self.config.session_data_path  # type: ignore
        self._api_response_cond = asyncio.Condition()
        self.jct: str = ""
        self.cookies = {}
        _path = f"{dirname(abspath(sys.argv[0]))}/{self.session_data_path}"
        if not os.path.exists(_path):
            os.mkdir(dirname(_path))
        if exists(_path):
            with open(_path) as f:
                self.cookies = json.load(f)
        user_cookies.update_cookies(self.cookies)
        if self.config.login:  # type: ignore
            logger.debug("Login enabled!")
            try:
                # 尝试登陆
                async with ClientSession(cookie_jar=user_cookies) as self.session:
                    success = await login(self.session)

                    if success:
                        self._uid = get_cookies("DedeUserID")
                        self.jct = get_cookies("bili_jct")

                        if self._uid == None or self.jct == None:  # noqa: E711
                            logger.error(
                                "Unable to get cookies, please check your cookies."
                            )
                            return
                        if not exists(_path):
                            for cookie in user_cookies:
                                self.cookies[cookie.key] = cookie.value

                            logger.debug(f"Stored cookies: {self.cookies}")
                            with open(_path, mode="w") as f:
                                json.dump(self.cookies, f)

                        await super().startup()
            except Exception as e:
                logger.error(e)
                return
        else:
            logger.debug("Login disabled!")
            await super().startup()

    async def websocket_connect(self):
        """创建正向 WebSocket 连接。"""

        logger.info("Trying to connect to WebSocket server...")
        host_server = self._host_server_list[
            self._retry_count % len(self._host_server_list)
        ]
        try:
            async with self.session.ws_connect(
                f'wss://{host_server["host"]}:{host_server["wss_port"]}/sub',
                receive_timeout=self._heartbeat_interval + 5,
            ) as self.websocket:
                await self._send_auth()
                self._heartbeat_timer_handle = asyncio.ensure_future(
                    self._start_heartbeat()
                )
                logger.success(f"Success to be invited to room {self.room_id}.")
                await self.handle_websocket()
        except Exception as e:
            logger.error(e)
            self._retry_count += 1
            await asyncio.sleep(self.reconnect_interval)
            await self.websocket_connect()

    async def handle_websocket_msg(self, msg: aiohttp.WSMessage):
        """处理 WebSocket 消息。"""
        logger.info(msg)
        if msg.type == aiohttp.WSMsgType.BINARY:
            try:
                data = msg.data  # await self.websocket.receive_bytes()
                logger.info(data)
                offset = 0
                while offset < len(data):
                    try:
                        header = HeaderTuple(*HEADER_STRUCT.unpack_from(data, offset))
                    except struct.error:
                        break
                    if header.operation == Operation.HEARTBEAT_REPLY:  # noqa: F405
                        popularity = int.from_bytes(
                            data[
                                offset + HEADER_STRUCT.size : offset
                                + HEADER_STRUCT.size
                                + 4
                            ],
                            "big",
                        )
                        await self._on_receive_popularity(popularity)
                    elif header.operation == Operation.SEND_MSG_REPLY:  # noqa: F405
                        body = data[
                            offset + HEADER_STRUCT.size : offset + header.pack_len
                        ]
                        if header.ver == WS_BODY_PROTOCOL_VERSION_DEFLATE:
                            self._loop = asyncio.get_event_loop()
                            body = await self._loop.run_in_executor(
                                None, zlib.decompress, body
                            )
                            # await self.handle_websocket_msg(body)
                            return
                        else:
                            try:
                                body = json.loads(body.decode("utf-8"))
                                data = body
                                logger.info(data)
                                data["post_type"] = data["cmd"].lower().split("_")[0]
                                data["message"] = data.get("msg_common") or ""
                                data["message_id"] = data.get("msg_id") or 0
                                data["group_id"] = data.get("roomid") or 0
                                data["time"] = data.get("send_time") or 0
                                await self.handle_bililive_event(data)  # type: ignore
                            except Exception:
                                logger.debug(f"body: {body}")
                                raise

                    elif header.operation == Operation.AUTH_REPLY:  # noqa: F405
                        await self.websocket.send_bytes(
                            self._make_packet({}, Operation.HEARTBEAT)  # noqa: F405
                        )

                    else:
                        body = data[
                            offset + HEADER_STRUCT.size : offset + header.pack_len
                        ]
                        logger.warning(
                            f"room {self.room_id,} 未知包类型：operation={header.operation, header, body}"
                        )

                    offset += header.pack_len
            except Exception as e:
                error_or_exception(
                    "WebSocket message parsing error, not BINARY:",
                    e,
                    self.bot.config.bot.log.verbose_exception,
                )
                async with self._api_response_cond:
                    self._api_response = msg.data
                    logger.warning(msg.data)
                    self._api_response_cond.notify_all()
        elif msg.type == aiohttp.WSMsgType.ERROR:
            logger.error(
                f"WebSocket connection closed "
                f"with exception {self.websocket.exception()!r}"
            )

    async def handle_bililive_event(self, data: Dict[str, Any]):
        logger.info(str(data))
        post_type = data.get("post_type")
        event_type = data.get(f"{post_type}_type")
        sub_type = data.get("sub_type", None)

        event_class = get_event_class(post_type, event_type, sub_type)
        bililive_event = event_class(adapter=self, **data)

        await self.handle_event(bililive_event)

    # 发送登录包
    async def _send_auth(self):
        auth_params = {
            "uid": self._uid or 0,  # 0: 游客
            "roomid": self.room_id,
            "protover": 2,
            "platform": "web",
            "clientver": "1.14.3",
            "type": 2,
        }
        await self.websocket.send_bytes(self._make_packet(auth_params, Operation.AUTH))  # noqa: F405

    @staticmethod
    def _make_packet(data, operation):
        body = json.dumps(data).encode("utf-8")
        header = HEADER_STRUCT.pack(
            HEADER_STRUCT.size + len(body), HEADER_STRUCT.size, 1, operation, 1
        )
        return header + body

    async def _start_heartbeat(self) -> None:
        """
        每30s一次心跳
        :return:
        """
        hb = "0000001f0010000100000002000000015b6f626a656374204f626a6563745d"
        try:
            while not self.bot.should_exit.is_set():
                if self.websocket.closed:
                    break
                await self.websocket.send_bytes(bytes.fromhex(hb))
                logger.debug("HeartBeat sent!")
                await asyncio.sleep(29)
        except Exception as e:
            logger.error(e)

    async def call_api(self, api: str, **params):
        """调用 bililive API。

        TODO: 因为基于OlivOS的那个OlivaBiliLive插件架构其实相当于一个小框架的缘故,
        所以要改的东西太多了，这里插个保留的接口...
        """

        ...

    async def send_danmu(self, **fields) -> bool:
        token = get_cookies("bili_jct")
        async with ClientSession(cookie_jar=user_cookies) as session:
            try:
                res = await _post(
                    session,
                    SEND_URL,
                    rnd=time.time(),
                    csrf=token,
                    csrf_token=token,
                    **fields,
                )
                return "data" in res
            except Exception as e:
                logger.warning(f"Send danmu failed: {e}")
                return False

    async def send(
        self,
        danmaku: str,
        fontsize: int = 25,
        color: int = 0xFFFFFF,
        pos: DanmakuPosition = DanmakuPosition.NORMAL,  # noqa: F405
    ) -> bool:
        # don't know what the hell is bubble
        return await self.send_danmu(
            msg=danmaku,
            fontsize=fontsize,
            color=color,
            pos=pos,
            roomid=self.room_id,
            bubble=0,
        )

    @abstractmethod
    async def _on_receive_popularity(self, popularity: int):
        pass


def rawData_to_jsonData(data: bytes):
    packetLen = int(data[:4].hex(), 16)
    ver = int(data[6:8].hex(), 16)
    op = int(data[8:12].hex(), 16)

    if len(data) > packetLen:  # 防止
        rawData_to_jsonData(data[packetLen:])
        data = data[:packetLen]

    if ver == 2:
        data = zlib.decompress(data[16:])
        return rawData_to_jsonData(data)

    if op == 5:
        try:
            jd = json.loads(data[16:].decode("utf-8", errors="ignore"))
            return jd
        except Exception:
            pass


async def login(session: ClientSession) -> bool:
    if get_cookies("bili_jct") != None:  # noqa: E711
        logger.info("Aleady login!")
        return True
    try:
        res = await _get(session, QRCODE_REQUEST_URL)
        ts = res["ts"]
        outdated = ts + 180 * 1000  # 180 秒後逾時
        authKey = res["data"]["oauthKey"]
        url = res["data"]["url"]
        qr = qrcode.QRCode()
        logger.info("请扫描下面的二维码进行登录... (或者到目录下寻找 qrcode.png)")
        qr.add_data(url)
        qr.print_ascii(invert=True)
        qr.make_image().save("qrcode.png")
        while True:
            await asyncio.sleep(5)
            if time.time() > outdated:
                logger.warning("Timeout!")
                return False  # 登入失敗
            res = await _post(session, CHECK_LOGIN_RESULT, oauthKey=authKey)
            if res["status"]:
                logger.success("login success!")
                return True
            else:
                code = res["data"]
                if code in [-1, -2]:
                    logger.warning(f'login failed: {res["message"]}')
                    return False
    except Exception as e:
        logger.warning(f"Something went wrong: {e}")
        return False
    finally:
        os.remove("qrcode.png")


def get_cookies(name: str) -> any:  # type: ignore
    for cookie in user_cookies:
        if cookie.key == name:
            return cookie.value
    return None


async def _get(session: ClientSession, url: str):
    async with session.get(url) as resp:
        resp.raise_for_status()
        data = await resp.json()
        logger.debug(data)
        if "code" in data and data["code"] != 0:
            raise Exception(data["message"] if "message" in data else data["code"])
        return data


async def _post(session: ClientSession, url: str, **data):
    form = aiohttp.FormData()
    for k, v in data.items():
        form.add_field(k, v)
    logger.debug(f"Sending POST: {url}, content: {data}")
    async with session.post(url, data=form) as resp:
        resp.raise_for_status()
        data = await resp.json()
        logger.debug(data)
        if "code" in data and data["code"] != 0:
            raise Exception(data["message"] if "message" in data else data["code"])
        return data



"""Kook 适配器事件。"""

import asyncio
import inspect
from enum import IntEnum
from collections import UserDict
from typing import (  # type: ignore
    TYPE_CHECKING,
    Any,
    Dict,
    List,
    Type,
    Tuple,
    Union,
    Literal,
    TypeVar,
    Optional,
)

from pydantic import Field, HttpUrl, BaseModel, validator

from iamai.event import Event

from .api import Role, User, Emoji, Guild, Channel
from .message import KookMessage

if TYPE_CHECKING:
    from .message import T_KookMSG

T_KookEvent = TypeVar("T_KookEvent", bound="KookEvent")


class ResultStore:
    _seq = 1
    _futures: Dict[Tuple[str, int], asyncio.Future] = {}
    _sn_map = {}

    @classmethod
    def set_sn(cls, self_id: str, sn: int) -> None:
        cls._sn_map[self_id] = sn

    @classmethod
    def get_sn(cls, self_id: str) -> int:
        return cls._sn_map.get(self_id, 0)


class AttrDict(UserDict):
    def __init__(self, data=None):
        initial = dict(data)  # type: ignore
        for k in initial:
            if isinstance(initial[k], dict):
                initial[k] = AttrDict(initial[k])  # type: ignore

        super().__init__(initial)

    def __getattr__(self, name):
        return self[name]


class PermissionOverwrite(BaseModel):
    role_id: Optional[int] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class PermissionUser(BaseModel):
    user: Optional[User] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class ChannelRoleInfo(BaseModel):
    """频道角色权限详情"""

    permission_overwrites: Optional[List[PermissionOverwrite]] = None
    """针对角色在该频道的权限覆写规则组成的列表"""
    permission_users: Optional[List[PermissionUser]] = None
    """针对用户在该频道的权限覆写规则组成的列表"""
    permission_sync: Optional[int] = None
    """权限设置是否与分组同步, 1 or 0"""


class Quote(BaseModel):
    """引用消息"""

    id_: Optional[str] = Field(None, alias="id")
    """引用消息 id"""
    type: Optional[int] = None
    """引用消息类型"""
    content: Optional[str] = None
    """引用消息内容"""
    create_at: Optional[int] = None
    """引用消息创建时间（毫秒）"""
    author: Optional[User] = None
    """作者的用户信息"""


class Attachments(BaseModel):
    """附加的多媒体数据"""

    type: Optional[str] = None
    """多媒体类型"""
    url: Optional[str] = None
    """多媒体地址"""
    name: Optional[str] = None
    """多媒体名"""
    size: Optional[int] = None
    """大小 单位（B）"""


class URL(BaseModel):
    url: Optional[str] = None
    """资源的 url"""


class Meta(BaseModel):
    page: Optional[int] = None
    page_total: Optional[int] = None
    page_size: Optional[int] = None
    total: Optional[int] = None


class ListReturn(BaseModel):
    meta: Optional[Meta] = None
    sort: Optional[Dict[str, Any]] = None


class BlackList(BaseModel):
    """黑名单"""

    user_id: Optional[str] = None
    """用户 id"""
    created_time: Optional[int] = None
    """加入黑名单的时间戳(毫秒)"""
    remark: Optional[str] = None
    """加入黑名单的原因"""
    user: Optional[User] = None
    """用户"""


class BlackListsReturn(ListReturn):
    """获取黑名单列表返回信息"""

    blacklists: Optional[List[BlackList]] = Field(None, alias="items")
    """黑名单列表"""


class MessageCreateReturn(BaseModel):
    """发送频道消息返回信息"""

    msg_id: Optional[str] = None
    """服务端生成的消息 id"""
    msg_timestamp: Optional[int] = None
    """消息发送时间(服务器时间戳)"""
    nonce: Optional[str] = None
    """随机字符串"""


class ChannelRoleReturn(BaseModel):
    """创建或更新频道角色权限返回信息"""

    role_id: Optional[int] = None
    user_id: Optional[str] = None
    allow: Optional[int] = None
    deny: Optional[int] = None


class GuildsReturn(ListReturn):
    guilds: Optional[List[Guild]] = Field(None, alias="items")


class ChannelsReturn(ListReturn):
    channels: Optional[List[Channel]] = Field(None, alias="items")


class GuildUsersRetrun(ListReturn):
    """服务器中的用户列表"""

    users: Optional[List[User]] = Field(None, alias="items")
    """用户列表"""
    user_count: Optional[int] = None
    """用户数量"""
    online_count: Optional[int] = None
    """在线用户数量"""
    offline_count: Optional[int] = None
    """离线用户数量"""


class Reaction(BaseModel):
    emoji: Optional[Emoji] = None
    count: Optional[int] = None
    me: Optional[bool] = None


class MentionInfo(BaseModel):
    mention_part: Optional[List[dict]] = None
    mention_role_part: Optional[List[dict]] = None
    channel_part: Optional[List[dict]] = None
    item_part: Optional[List[dict]] = None


class BaseMessage(BaseModel):
    id_: Optional[str] = Field(None, alias="id")
    """消息 ID"""
    type: Optional[int] = None
    """消息类型"""
    content: Optional[str] = None
    """消息内容"""
    embeds: Optional[List[dict]] = None
    """超链接解析数据"""
    attachments: Optional[Union[bool, Attachments]] = None
    """附加的多媒体数据"""
    create_at: Optional[int] = None
    """创建时间"""
    updated_at: Optional[int] = None
    """更新时间"""
    reactions: Optional[List[Reaction]] = None
    """回应数据"""
    image_name: Optional[str] = None
    """"""
    read_status: Optional[bool] = None
    """是否已读"""
    quote: Optional[Quote] = None
    """引用数据"""
    mention_info: Optional[MentionInfo] = None
    """引用特定用户或特定角色的信息"""


class ChannelMessage(BaseMessage):
    """频道消息"""

    author: Optional[User] = None
    mention: Optional[List[Any]] = None
    mention_all: Optional[bool] = None
    mention_roles: Optional[List[Any]] = None
    mention_here: Optional[bool] = None


class DirectMessage(BaseMessage):
    """私聊消息"""

    author_id: Optional[str] = None
    """作者的用户 ID"""
    from_type: Optional[int] = None
    """from_type"""
    msg_icon: Optional[str] = None
    """msg_icon"""


class ChannelMessagesReturn(BaseModel):
    """获取私信聊天消息列表返回信息"""

    direct_messages: Optional[List[ChannelMessage]] = Field(None, alias="items")


class DirectMessagesReturn(BaseModel):
    """获取私信聊天消息列表返回信息"""

    direct_messages: Optional[List[DirectMessage]] = Field(None, alias="items")


class ReactionUser(User):
    reaction_time: Optional[int] = None


class TargetInfo(BaseModel):
    """私聊会话 目标用户信息"""

    id_: Optional[str] = Field(None, alias="id")
    """目标用户 ID"""
    username: Optional[str] = None
    """目标用户名"""
    online: Optional[bool] = None
    """是否在线"""
    avatar: Optional[str] = None
    """头像图片链接"""


class UserChat(BaseModel):
    """私聊会话"""

    code: Optional[str] = None
    """私信会话 Code"""
    last_read_time: Optional[int] = None
    """上次阅读消息的时间 (毫秒)"""
    latest_msg_time: Optional[int] = None
    """最新消息时间 (毫秒)"""
    unread_count: Optional[int] = None
    """未读消息数"""
    target_info: Optional[TargetInfo] = None
    """目标用户信息"""


class UserChatsReturn(ListReturn):
    """获取私信聊天会话列表返回信息"""

    user_chats: Optional[List[UserChat]] = Field(None, alias="items")
    """私聊会话列表"""


class RolesReturn(ListReturn):
    """获取服务器角色列表返回信息"""

    roles: Optional[List[Role]] = Field(None, alias="items")
    """服务器角色列表"""


class GuilRoleReturn(BaseModel):
    """赋予或删除用户角色返回信息"""

    user_id: Optional[str] = None
    """用户 id"""
    guild_id: Optional[str] = None
    """服务器 id"""
    roles: Optional[List[int]] = None
    """角色 id 的列表"""


class IntimacyImg(BaseModel):
    """形象图片的总列表"""

    id_: Optional[str] = Field(None, alias="id")
    """	形象图片的 id"""
    url: Optional[str] = None
    """形象图片的地址"""


class IntimacyIndexReturn(BaseModel):
    """获取用户亲密度返回信息"""

    img_url: Optional[str] = None
    """机器人给用户显示的形象图片地址"""
    social_info: Optional[str] = None
    """机器人显示给用户的社交信息"""
    last_read: Optional[int] = None
    """用户上次查看的时间戳"""
    score: Optional[int] = None
    """亲密度，0-2200"""
    img_list: Optional[List[IntimacyImg]] = None
    """形象图片的总列表"""


class GuildEmoji(BaseModel):
    """服务器表情"""

    name: Optional[str] = None
    """表情的名称"""
    id_: Optional[str] = Field(None, alias="id")
    """表情的 ID"""
    user_info: Optional[User] = None
    """上传用户"""


class GuildEmojisReturn(ListReturn):
    """获取服务器表情列表返回信息"""

    roles: Optional[List[GuildEmoji]] = Field(None, alias="items")
    """服务器表情列表"""


class Invite(BaseModel):
    """邀请信息"""

    guild_id: Optional[str] = None
    """服务器 id"""
    channel_id: Optional[str] = None
    """频道 id"""
    url_code: Optional[str] = None
    """url code"""
    url: Optional[str] = None
    """地址"""
    user: Optional[User] = None
    """用户"""


class InvitesReturn(ListReturn):
    """获取邀请列表返回信息"""

    roles: Optional[List[Invite]] = Field(None, alias="items")
    """邀请列表"""


class EventTypes(IntEnum):
    """
    事件主要格式
    Kook 协议事件，字段与 Kook 一致。各事件字段参考 `Kook 文档`

    .. Kook 文档:
        https://developer.kookapp.cn/doc/event/event-introduction#事件主要格式
    """

    text = 1
    image = 2
    video = 3
    file = 4
    audio = 8
    kmarkdown = 9
    card = 10
    sys = 255


class SignalTypes(IntEnum):
    """
    信令类型
    Kook 协议信令，字段与 Kook 一致。各事件字段参考 `Kook 文档`

    .. Kook 文档:
        https://developer.kookapp.cn/doc/websocket#信令格式
    """

    EVENT = 0
    HELLO = 1
    PING = 2
    PONG = 3
    RESUME = 4
    RECONNECT = 5
    RESUME_ACK = 6
    SYS = 255


class Attachment(BaseModel):
    type: str
    name: str
    url: HttpUrl
    file_type: Optional[str] = Field(None)
    size: Optional[int] = Field(None)
    duration: Optional[float] = Field(None)
    width: Optional[int] = Field(None)
    hight: Optional[int] = Field(None)


class Extra(BaseModel):
    type_: Union[int, str] = Field(None, alias="type")
    guild_id: Optional[str] = Field(None)
    channel_name: Optional[str] = Field(None)
    mention: Optional[List[str]] = Field(None)
    mention_all: Optional[bool] = Field(None)
    mention_roles: Optional[List[str]] = Field(None)
    mention_here: Optional[bool] = Field(None)
    author: Optional[User] = Field(None)
    body: Optional[AttrDict] = Field(None)
    attachments: Optional[Attachment] = Field(None)
    code: Optional[str] = Field(None)

    @validator("body", pre=True)
    def convert_body(cls, v):
        if v is None:
            return None

        if not isinstance(v, dict):
            raise TypeError("body must be dict")
        if not isinstance(v, AttrDict):
            v = AttrDict(v)
        return v

    class Config:
        arbitrary_types_allowed = True


class OriginEvent(Event["KookAdapter"]):
    """为了区分信令中非Event事件，增加了前置OriginEvent"""

    __event__ = ""

    post_type: str


class Kmarkdown(BaseModel):
    raw_content: str
    mention_part: list
    mention_role_part: list


class EventMessage(BaseModel):
    type: Union[int, str]
    guild_id: Optional[str]
    channel_name: Optional[str]
    mention: Optional[List]
    mention_all: Optional[bool]
    mention_roles: Optional[List]
    mention_here: Optional[bool]
    nav_channels: Optional[List]
    author: User

    kmarkdown: Optional[Kmarkdown]

    code: Optional[str] = None
    attachments: Optional[Attachment] = None

    content: KookMessage


class KookEvent(OriginEvent):
    """
    事件主要格式，来自 d 字段
    Kook 协议事件，字段与 Kook 一致。各事件字段参考 `Kook 文档`

    .. Kook 文档:
        https://developer.kookapp.cn/doc/event/event-introduction
    """

    __event__ = ""
    channel_type: Literal["PERSON", "GROUP"]
    type_: int = Field(alias="type")
    """1:文字消息\n2:图片消息\n3:视频消息\n4:文件消息\n8:音频消息\n9:KMarkdown\n10:card消息\n255:系统消息\n其它的暂未开放"""
    target_id: str
    """
    发送目的\n
    频道消息类时, 代表的是频道 channel_id\n
    如果 channel_type 为 GROUP 组播且 type 为 255 系统消息时，则代表服务器 guild_id"""
    author_id: Optional[str] = None
    content: KookMessage
    msg_id: str
    msg_timestamp: int
    nonce: str
    extra: Extra
    user_id: str

    post_type: str
    self_id: Optional[str] = None  # onebot兼容


# Message Events
class MessageEvent(KookEvent):
    """消息事件"""

    __event__ = "message"

    post_type: Literal["message"] = "message"
    message_type: str  # group private 其实是person
    sub_type: str
    event: EventMessage

    def __repr__(self) -> str:
        return f'Event<{self.post_type}>: "{self.content}"'

    def get_plain_text(self) -> str:
        """获取消息的纯文本内容。

        Returns:
            消息的纯文本内容。
        """
        return self.content.get_plain_text()  # type: ignore

    async def reply(self, msg: "T_KookMSG") -> Dict[str, Any]:
        """回复消息。

        Args:
            msg: 回复消息的内容，同 `call_api()` 方法。

        Returns:
            API 请求响应。
        """
        raise NotImplementedError


class PrivateMessageEvent(MessageEvent):
    """私聊消息"""

    __event__ = "message.private"
    message_type: Literal["private"]

    async def reply(self, msg: "T_KookMSG") -> Dict[str, Any]:
        return await self.adapter.call_api(
            api="direct-message/create", target_id=self.author_id, content=msg
        )


class ChannelMessageEvent(MessageEvent):
    """公共频道消息"""

    __event__ = "message.group"
    message_type: Literal["group"]
    group_id: str

    async def reply(self, msg: "T_KookMSG") -> Dict[str, Any]:
        return await self.adapter.call_api(
            "message/create", target_id=self.target_id, content=msg
        )


# Notice Events
class NoticeEvent(KookEvent):
    """通知事件"""

    __event__ = "notice"
    post_type: Literal["notice"]
    notice_type: str

    def __repr__(self) -> str:
        return f'Event<{self.post_type}>: "{self.content}"'


# Channel Events
class ChannelNoticeEvent(NoticeEvent):
    """频道消息事件"""

    __event__ = "notice"
    group_id: int


class ChannelAddReactionEvent(ChannelNoticeEvent):
    """频道内用户添加 reaction"""

    __event__ = "notice.added_reaction"
    notice_type: Literal["added_reaction"]


class ChannelDeletedReactionEvent(ChannelNoticeEvent):
    """频道内用户删除 reaction"""

    __event__ = "notice.deleted_reaction"
    notice_type: Literal["deleted_reaction"]


class ChannelUpdatedMessageEvent(ChannelNoticeEvent):
    """频道消息更新"""

    __event__ = "notice.updated_message"
    notice_type: Literal["updated_message"]


class ChannelDeleteMessageEvent(ChannelNoticeEvent):
    """频道消息被删除"""

    __event__ = "notice.deleted_message"
    notice_type: Literal["deleted_message"]


class ChannelAddedEvent(ChannelNoticeEvent):
    """新增频道"""

    __event__ = "notice.added_channel"
    notice_type: Literal["added_channel"]


class ChannelUpdatedEvent(ChannelNoticeEvent):
    """修改频道信息"""

    __event__ = "notice.updated_channel"
    notice_type: Literal["updated_channel"]


class ChannelDeleteEvent(ChannelNoticeEvent):
    """删除频道"""

    __event__ = "notice.deleted_channel"
    notice_type: Literal["deleted_channel"]


class ChannelPinnedMessageEvent(ChannelNoticeEvent):
    """新增频道置顶消息"""

    __event__ = "notice.pinned_message"
    notice_type: Literal["pinned_message"]


class ChannelUnpinnedMessageEvent(ChannelNoticeEvent):
    """取消频道置顶消息"""

    __event__ = "notice.unpinned_message"
    notice_type: Literal["unpinned_message"]


# Private Events
class PrivateNoticeEvent(NoticeEvent):
    "私聊消息事件"


class PrivateUpdateMessageEvent(PrivateNoticeEvent):
    """私聊消息更新"""

    __event__ = "notice.updated_private_message"
    notice_type: Literal["updated_private_message"]


class PrivateDeleteMessageEvent(PrivateNoticeEvent):
    """私聊消息删除"""

    __event__ = "notice.deleted_private_message"
    notice_type: Literal["deleted_private_message"]


class PrivateAddReactionEvent(PrivateNoticeEvent):
    """私聊内用户添加 reaction"""

    __event__ = "notice.private_added_reaction"
    notice_type: Literal["private_added_reaction"]


class PrivateDeleteReactionEvent(PrivateNoticeEvent):
    """私聊内用户取消 reaction"""

    __event__ = "notice.private_deleted_reaction"
    notice_type: Literal["private_deleted_reaction"]


# Guild Events
class GuildNoticeEvent(NoticeEvent):
    """服务器相关事件"""

    group_id: int

    def get_guild_id(self):
        return self.target_id  # type: ignore


# Guild Member Events
class GuildMemberNoticeEvent(GuildNoticeEvent):
    """服务器成员相关事件"""

    pass


class GuildMemberIncreaseNoticeEvent(GuildMemberNoticeEvent):
    """新成员加入服务器"""

    __event__ = "notice.joined_guild"
    notice_type: Literal["joined_guild"]


class GuildMemberDecreaseNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员退出"""

    __event__ = "notice.exited_guild"
    notice_type: Literal["exited_guild"]


class GuildMemberUpdateNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员信息更新(修改昵称)"""

    __event__ = "notice.updated_guild_member"
    notice_type: Literal["updated_guild_member"]


class GuildMemberOnlineNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员上线"""

    __event__ = "notice.guild_member_online"
    notice_type: Literal["guild_member_online"]


class GuildMemberOfflineNoticeEvent(GuildMemberNoticeEvent):
    """服务器成员下线"""

    __event__ = "notice.guild_member_offline"
    notice_type: Literal["guild_member_offline"]


# Guild Role Events
class GuildRoleNoticeEvent(GuildNoticeEvent):
    """服务器角色相关事件"""


class GuildRoleAddNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.added_role"
    notice_type: Literal["added_role"]


class GuildRoleDeleteNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.deleted_role"
    notice_type: Literal["deleted_role"]


class GuildRoleUpdateNoticeEvent(GuildRoleNoticeEvent):
    """服务器角色增加"""

    __event__ = "notice.updated_role"
    notice_type: Literal["updated_role"]


# Guild Events
class GuildUpdateNoticeEvent(GuildNoticeEvent):
    """服务器信息更新"""

    __event__ = "notice.updated_guild"
    notice_type: Literal["updated_guild"]


class GuildDeleteNoticeEvent(GuildNoticeEvent):
    """服务器删除"""

    __event__ = "notice.deleted_guild"
    notice_type: Literal["deleted_guild"]


class GuildAddBlockListNoticeEvent(GuildNoticeEvent):
    """服务器封禁用户"""

    __event__ = "notice.added_block_list"
    notice_type: Literal["added_block_list"]


class GuildDeleteBlockListNoticeEvent(GuildNoticeEvent):
    """服务器取消封禁用户"""

    __event__ = "notice.deleted_block_list"
    notice_type: Literal["deleted_block_list"]


# User Events
class UserNoticeEvent(NoticeEvent):
    """用户相关事件列表"""

    group_id: int


class UserJoinAudioChannelNoticeEvent(UserNoticeEvent):
    """用户加入语音频道"""

    __event__ = "notice.joined_channel"
    notice_type: Literal["joined_channel"]


class UserJoinAudioChannelEvent(UserNoticeEvent):
    """用户退出语音频道"""

    __event__ = "notice.exited_channel"
    notice_type: Literal["exited_channel"]


class UserInfoUpdateNoticeEvent(UserNoticeEvent):
    """
    用户信息更新

    该事件与服务器无关, 遵循以下条件:
        - 仅当用户的 用户名 或 头像 变更时
        - 仅通知与该用户存在关联的用户或 Bot
            a. 存在聊天会话
            b. 双方好友关系
    """

    __event__ = "notice.user_updated"
    notice_type: Literal["user_updated"]


class SelfJoinGuildNoticeEvent(NoticeEvent):
    """
    自己新加入服务器

    当自己被邀请或主动加入新的服务器时, 产生该事件
    """

    __event__ = "notice.self_joined_guild"
    notice_type: Literal["self_joined_guild"]
    user_id: str
    group_id: int


class SelfExitGuildNoticeEvent(NoticeEvent):
    """
    自己退出服务器

    当自己被踢出服务器或被拉黑或主动退出服务器时, 产生该事件
    """

    __event__ = "notice.self_exited_guild"
    notice_type: Literal["self_exited_guild"]
    user_id: str
    group_id: int


class CartBtnClickNoticeEvent(NoticeEvent):
    """
    Card 消息中的 Button 点击事件
    """

    __event__ = "notice.message_btn_click"
    notice_type: Literal["message_btn_click"]
    user_id: str
    group_id: int


# Meta Events
class MetaEvent(OriginEvent):
    """元事件"""

    __event__ = "meta_event"
    post_type: Literal["meta_event"]
    meta_event_type: str


class LifecycleMetaEvent(MetaEvent):
    """生命周期元事件"""

    __event__ = "meta_event.lifecycle"
    meta_event_type: Literal["lifecycle"]
    sub_type: str


class HeartbeatMetaEvent(MetaEvent):
    """心跳元事件"""

    __event__ = "meta_event.heartbeat"
    meta_event_type: Literal["heartbeat"]
    sub_type: str


# 事件类映射
_kook_events = {
    model.__event__: model
    for model in globals().values()
    if inspect.isclass(model) and issubclass(model, OriginEvent)
}


def get_event_class(
    post_type: str, event_type: str, sub_type: Optional[str] = None
) -> Type[T_KookEvent]:  # type: ignore
    """根据接收到的消息类型返回对应的事件类。

    Args:
        post_type: 请求类型。
        event_type: 事件类型。
        sub_type: 子类型。

    Returns:
        对应的事件类。
    """
    if sub_type is None:
        return _kook_events[".".join((post_type, event_type))]  # type: ignore
    return (
        _kook_events.get(".".join((post_type, event_type, sub_type)))
        or _kook_events[".".join((post_type, event_type))]
    )  # type: ignore



"""red 协议适配器。

本适配器适配了 red 协议。
协议详情请参考: [RedProtocol](https://chrononeko.github.io/QQNTRedProtocol/) 。
"""

import os
import json
import asyncio
from uu import Error
from functools import partial
from typing import TYPE_CHECKING, Any, Dict, Literal

import yaml
import aiohttp
from itsdangerous import exc  # noqa: F401

from iamai.log import logger
from iamai.adapter.utils import WebSocketAdapter

from .api import HANDLE
from .exceptions import *  # noqa: F403
from .message import RedMessage
from .config import USER_CONFIG, Config
from .event import MsgType, RedEvent, get_event_class  # noqa: F401

if TYPE_CHECKING:
    from .message import T_RedMSG  # type: ignore


__all__ = ["RedAdapter"]


class RedAdapter(WebSocketAdapter[RedEvent, Config]):
    """Red 协议适配器。"""

    name: str = "red"
    Config = Config
    config: Config

    _api_response: Dict[Any, Any]
    _api_response_cond: asyncio.Condition = None  # type: ignore
    _api_id: int = 0

    def __getattr__(self, item: Any):
        return partial(self.call_api, item)

    async def startup(self):
        """初始化适配器。"""
        self.adapter_type = "ws"
        self.host = self.config.host
        self.port = self.config.port
        self.access_token = self.config.access_token
        self.reconnect_interval = self.config.reconnect_interval
        self._api_response_cond = asyncio.Condition()
        if self.config.auto_fill:
            logger.info("Auto Detecting Chronocat Config...")
            self.chronocat_config = self.get_red_config()
            if not self.chronocat_config:
                raise Error("Can not parse or find Chronocat Config file!")
            logger.success("Succeed to Parse Chronocat Config.")
            servers = self.chronocat_config["servers"]
            red = servers[0] if servers[0]["type"] == "red" else servers[1]
            self.port = red["port"]
            self.access_token = red["token"]
        logger.debug(f"token: {self.access_token}")
        await super().startup()

    async def websocket_connect(self):
        """创建正向 WebSocket 连接。"""
        logger.info("Tying to connect to WebSocket server...")
        async with self.session.ws_connect(
            f"ws://{self.host}:{self.port}/",
            headers=(
                {"Authorization": f"Bearer {self.access_token}"}
                if self.access_token
                else None
            ),
        ) as self.websocket:
            connect = {
                "type": "meta::connect",
                "payload": {"token": self.access_token},
            }
            await self.websocket.send_json(connect)
            await self.handle_websocket()

    async def handle_websocket_msg(self, msg: aiohttp.WSMessage):
        """处理 WebSocket 消息。"""
        msg_dict = json.loads(msg.data)
        msg_data = msg_dict["payload"]
        if self.config.show_raw:
            logger.info(msg_data)
        if msg_dict["type"] == "meta::connect":
            self.self_id = msg_data.get("authData").get("account")
            logger.success(
                f"WebSocket connection "
                f"from {msg_data.get('name')}({msg_data.get('version')}) Bot {self.self_id} accepted!"
            )
        elif msg_dict["type"] == "message::recv":
            msg_data = msg_data[0]
            try:
                data = msg_data
                if msg_data.get("chatType", None):
                    data["post_type"] = "message"
                    data["message_type"] = (
                        "private" if msg_data["chatType"] == 1 else "group"
                    )

                    if data["message_type"] == "group":
                        data["group_id"] = msg_data.get("peerUid")
                        data["sub_type"] = "normal"
                    if data["message_type"] == "private":
                        data["user_id"] = msg_data.get("peerUid")
                        data["sub_type"] = "group"
                    data["timestamp"] = msg_data.get("msgTime")
                    data["nick_name"] = msg_data.get("sendNickName")
                    data["msgId"] = msg_data.get("msgId")
                    try:
                        data["message"] = (
                            msg_data.get("elements")[0]
                            .get("textElement")
                            .get("content")
                        )
                    except:  # noqa: E722
                        data["message"] = msg_data.get("elements")[0].get("summary")
                    logger.info(f"Event Received: {data}")
                # elif (
                #     msg_data.get("msgType") == MsgType.system and msg_data.get("sendType") == 3
                # ):
                #     data["post_type"] = "notice"
                #     if sub_type := msg_data["elements"][0]["grayTipElement"][
                #         "groupElement"
                #     ]:
                #         if sub_type["type"] == 1:
                #             data["notice_type"] = "member_add"
                #         if sub_type["type"] == 8:
                #             data["notice_type"] = "member_mute"
                #         if sub_type["type"] == 5:
                #             data["notice_type"] = "group_name_update"
                # if xml_type := msg_data["elements"][0]["grayTipElement"]["xmlElement"]:
                #     if (
                #         xml_type["subElementType"] == 12
                #         and xml_type["busiType"] == "1"
                #         and xml_type["busiId"] == "10145"
                #     ):
                #         data["notice_type"] = "member_unmute"
                await self.handle_red_event(data)
            except Exception as e:
                logger.error(f"Event Handled Error with {e!r}")
        elif msg.type == aiohttp.WSMsgType.ERROR:
            logger.error(
                f"Websocket connection closed "
                f"with exception {self.websocket.exception()!r}"
            )

    async def handle_red_event(self, msg: Dict[str, Any]):
        """处理 red 事件。

        Args:
            msg: 接收到的信息。
        """
        post_type = msg.get("post_type")
        event_type = msg.get(f"{post_type}_type")
        sub_type = msg.get("sub_type", None)
        event_class = get_event_class(post_type, event_type, sub_type)  # type: ignore
        red_event = event_class(adapter=self, **msg)

        await self.handle_event(red_event)

    async def call_api(self, api: str, **params) -> Dict[str, Any]:  # type: ignore
        url = f"http://{self.host}/{self.port}/api"

        if api not in HANDLE:
            raise ValueError(f"API '{api}' is not supported.")

        sender = HANDLE[api](params)

        async with aiohttp.ClientSession() as session:
            endpoint, method, payload = sender(params)
            async with session.request(
                method, url=f"{url}/{endpoint}", json=payload
            ) as response:
                return await response.json()

    @staticmethod
    def get_red_config():
        if not os.path.exists(USER_CONFIG):
            return None
        with open(USER_CONFIG, encoding="utf-8") as f:
            chronocat_config = yaml.safe_load(f.read())
        return chronocat_config

    async def send(
        self, elements: "T_RedMSG", chatType: Literal["private", "group"], peerUin: int
    ) -> Dict[str, Any]:
        """发送消息，调用 send_message API 发送消息。"""
        if chatType == "private":
            return await self.send_message(
                chatType=1, peerUin=peerUin, elements=RedMessage(elements)
            )
        elif chatType == "group":
            return await self.send_message(
                chatType=2, peerUin=peerUin, elements=RedMessage(elements)
            )
        raise TypeError('message_type must be "private" or "group"')



import click
import asyncio

@click.group()
def cli():
    pass

@cli.command()
@click.argument("plugin_name", required=True)
def install(plugin_name):
    """安装iamai插件"""
    click.echo(f"正在安装 {plugin_name} 插件...")

@cli.command()
def version():
    """显示iamai版本"""
    click.echo("iamai版本: 0.1.5")

def cli_func(*args):
    cli(*args)


"""Bililive 适配器配置。"""

from typing import Literal

from iamai.config import ConfigModel


class Config(ConfigModel):
    """Bililive 配置类，将在适配器被加载时被混入到机器人主配置中。

    Attributes:
        adapter_type: 适配器类型，需要和协议端配置相同。
        reconnect_interval: 重连等待时间。
        api_timeout: 进行 API 调用时等待返回响应的超时时间。
        show_raw: 是否显示原始数据，默认为 False，不显示。
        session_data_path: session 数据文件路径, 默认为 "data/session.token"。
        report_self_message: 是否上报自己发送的消息，默认为 False，不上报。
        room_id: 监听的房间号列表，默认为 [0]）。
        ssl: 是否使用 SSL，默认为 True，使用。
    """

    __config_name__ = "bililive"
    adapter_type: Literal["ws"] = "ws"
    reconnect_interval: int = 3
    api_timeout: int = 1000
    session_data_path: str = "data/session.token"
    show_raw: bool = False
    report_self_message: bool = False
    room_id: int = 0
    login: bool = True



from iamai import Plugin
from iamai.log import logger
from iamai.models.BM25 import BM25
from iamai.adapter.kook.message import (
    KookMessage,
    KookMessageSegment,
)

documents = [
    "管制品兑换-远程武器-手枪-等级1".split("-"),
    "唐肆-技能-舞枪戈".split("-"),
    "五金店-道具-钉枪-等级0".split("-"),
    "五金店-道具-扳手-等级0".split("-"),
    "武器店-组件-民用自卫手枪弹夹-等级1".split("-"),
]
bm25 = BM25(documents)


class EvalKook(Plugin):
    global bm25

    async def handle(self) -> None:
        msg = KookMessage()
        msg += KookMessageSegment.text("jijijijiji")
        await self.event.reply(msg)
        logger.info(list(self.event))
        # await self.event.adapter.call_api(
        #     api="message/create", target_id=self.event.group_id, content=KookMessageSegment.KMarkdown('**hi**')
        # )
        # await self.event.reply(bm25.get_score(query))
        await self.event.adapter.call_api(
            api="message/create",
            target_id=self.event.group_id,
            content=KookMessageSegment.KMarkdown("**hi**"),
        )

    async def rule(self) -> bool:
        if self.event.adapter.name != "kook":
            return False
        return self.event.content.startswith(".test")



from iamai import Plugin


class Halloiamai(Plugin):
    async def handle(self) -> None:
        await self.event.reply("Hello, iamai!")

    async def rule(self) -> bool:
        if self.event.adapter.name != "dingtalk":
            return False
        return str(self.event.message).strip().lower() == "hello"



"""A utility used internally by iamai."""

import asyncio
import importlib
import inspect
import json
import os
import os.path
import sys
import traceback
from abc import ABC
from contextlib import asynccontextmanager
from functools import partial
from importlib.abc import MetaPathFinder
from importlib.machinery import ModuleSpec, PathFinder
from types import GetSetDescriptorType, ModuleType
from typing import (
    TYPE_CHECKING,
    Any,
    AsyncGenerator,
    Awaitable,
    Callable,
    ClassVar,
    ContextManager,
    Coroutine,
    Dict,
    List,
    Optional,
    Sequence,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)
from typing_extensions import ParamSpec, TypeAlias, TypeGuard

from pydantic import BaseModel

from .config import ConfigModel
from .typing import EventT

if TYPE_CHECKING:
    from os import PathLike

__all__ = [
    "ModulePathFinder",
    "is_config_class",
    "get_classes_from_module",
    "get_classes_from_module_name",
    "PydanticEncoder",
    "samefile",
    "sync_func_wrapper",
    "sync_ctx_manager_wrapper",
    "wrap_get_func",
    "get_annotations",
]

_T = TypeVar("_T")
_P = ParamSpec("_P")
_R = TypeVar("_R")
_TypeT = TypeVar("_TypeT", bound=Type[Any])

StrOrBytesPath: TypeAlias = Union[str, bytes, "PathLike[str]", "PathLike[bytes]"]


class ModulePathFinder(MetaPathFinder):
    """Meta path finder for finding iamai components."""

    path: ClassVar[List[str]] = []

    def find_spec(
        self,
        fullname: str,
        path: Optional[Sequence[str]] = None,
        target: Optional[ModuleType] = None,
    ) -> Union[ModuleSpec, None]:
        """Used to find the ``spec`` of a specified module."""
        if path is None:
            path = []
        return PathFinder.find_spec(fullname, self.path + list(path), target)


def is_config_class(config_class: Any) -> TypeGuard[Type[ConfigModel]]:
    """Determine whether an object is a configuration class.

    Args:
        config_class: The object to be judged.

    Returns:
        Returns whether it is a configuration class.
    """
    return (
        inspect.isclass(config_class)
        and issubclass(config_class, ConfigModel)
        and isinstance(getattr(config_class, "__config_name__", None), str)
        and ABC not in config_class.__bases__
        and not inspect.isabstract(config_class)
    )


def get_classes_from_module(module: ModuleType, super_class: _TypeT) -> List[_TypeT]:
    """Find a class of the specified type from the module.

    Args:
        module: Python module.
        super_class: The superclass of the class to be found.

    Returns:
        Returns a list of classes that meet the criteria.
    """
    classes: List[_TypeT] = []
    for _, module_attr in inspect.getmembers(module, inspect.isclass):
        if (
            (inspect.getmodule(module_attr) or module) is module
            and issubclass(module_attr, super_class)
            and module_attr != super_class
            and ABC not in module_attr.__bases__
            and not inspect.isabstract(module_attr)
        ):
            classes.append(cast(_TypeT, module_attr))
    return classes


def get_classes_from_module_name(
    name: str, super_class: _TypeT, *, reload: bool = False
) -> List[Tuple[_TypeT, ModuleType]]:
    """Find a class of the specified type from the module with the specified name.

    Args:
        name: module name, the format is the same as the Python ``import`` statement.
        super_class: The superclass of the class to be found.
        reload: Whether to reload the module.

    Returns:
        Returns a list of tuples consisting of classes and modules that meet the criteria.

    Raises:
        ImportError: An error occurred while importing the module.
    """
    try:
        importlib.invalidate_caches()
        module = importlib.import_module(name)
        if reload:
            importlib.reload(module)
        return [(x, module) for x in get_classes_from_module(module, super_class)]
    except KeyboardInterrupt:
        # Do not capture KeyboardInterrupt
        # Catching KeyboardInterrupt will prevent the user from closing Python when the module being imported is stuck in an infinite loop
        raise
    except BaseException as e:
        raise ImportError(e, traceback.format_exc()) from e


class PydanticEncoder(json.JSONEncoder):
    """``JSONEncoder`` class for parsing ``pydantic.BaseModel``."""

    def default(self, o: Any) -> Any:
        """Returns a serializable object of ``o``."""
        if isinstance(o, BaseModel):
            return o.model_dump(mode="json")
        return super().default(o)


def samefile(path1: StrOrBytesPath, path2: StrOrBytesPath) -> bool:
    """A simple wrapper around ``os.path.samefile``.

    Args:
        path1: path1.
        path2: path 2.

    Returns:
        If two paths point to the same file or directory.
    """
    try:
        return path1 == path2 or os.path.samefile(path1, path2)  # noqa: PTH121
    except OSError:
        return False


def sync_func_wrapper(
    func: Callable[_P, _R], *, to_thread: bool = False
) -> Callable[_P, Coroutine[None, None, _R]]:
    """Wrap a synchronous function as an asynchronous function.

    Args:
        func: synchronous function to be packaged.
        to_thread: Whether to run the synchronization function in a separate thread. Defaults to ``False``.

    Returns:
        Asynchronous functions.
    """
    if to_thread:

        async def _wrapper(*args: _P.args, **kwargs: _P.kwargs) -> _R:
            loop = asyncio.get_running_loop()
            func_call = partial(func, *args, **kwargs)
            return await loop.run_in_executor(None, func_call)

    else:

        async def _wrapper(*args: _P.args, **kwargs: _P.kwargs) -> _R:
            return func(*args, **kwargs)

    return _wrapper


@asynccontextmanager
async def sync_ctx_manager_wrapper(
    cm: ContextManager[_T], *, to_thread: bool = False
) -> AsyncGenerator[_T, None]:
    """Wrap a synchronous context manager into an asynchronous context manager.

    Args:
        cm: The synchronization context manager to be wrapped.
        to_thread: Whether to run the synchronization function in a separate thread. Defaults to ``False``.

    Returns:
        Asynchronous context manager.
    """
    try:
        yield await sync_func_wrapper(cm.__enter__, to_thread=to_thread)()
    except Exception as e:
        if not await sync_func_wrapper(cm.__exit__, to_thread=to_thread)(
            type(e), e, e.__traceback__
        ):
            raise
    else:
        await sync_func_wrapper(cm.__exit__, to_thread=to_thread)(None, None, None)


def wrap_get_func(
    func: Optional[Callable[[EventT], Union[bool, Awaitable[bool]]]],
) -> Callable[[EventT], Awaitable[bool]]:
    """Wrap the parameters accepted by the ``get()`` function into an asynchronous function.

    Args:
        func: The parameters accepted by the ``get()`` function.

    Returns:
        Asynchronous functions.
    """
    if func is None:
        return sync_func_wrapper(lambda _: True)
    if not asyncio.iscoroutinefunction(func):
        return sync_func_wrapper(func)  # type: ignore
    return func


if sys.version_info >= (3, 10):  # pragma: no cover
    from inspect import get_annotations
else:  # pragma: no cover

    def get_annotations(
        obj: Union[Callable[..., object], Type[Any], ModuleType],
    ) -> Dict[str, Any]:
        """Compute the annotation dictionary of an object.

        Args:
            obj: A callable object, class, or module.

        Raises:
            TypeError: ``obj`` is not a callable object, class or module.
            ValueError: Object's ``__annotations__`` is not a dictionary or ``None``.

        Returns:
            Annotation dictionary for objects.
        """
        ann: Union[Dict[str, Any], None]

        if isinstance(obj, type):
            # class
            obj_dict = getattr(obj, "__dict__", None)
            if obj_dict and hasattr(obj_dict, "get"):
                ann = obj_dict.get("__annotations__", None)
                if isinstance(ann, GetSetDescriptorType):
                    ann = None
            else:
                ann = None
        elif isinstance(obj, ModuleType) or callable(obj):
            # this includes types.ModuleType, types.Function, types.BuiltinFunctionType,
            # types.BuiltinMethodType, functools.partial, functools.singledispatch,
            # "class funclike" from Lib/test/test_inspect... on and on it goes.
            ann = getattr(obj, "__annotations__", None)
        else:
            raise TypeError(f"{obj!r} is not a module, class, or callable.")

        if ann is None:
            return {}

        if not isinstance(ann, dict):
            raise ValueError(  # noqa: TRY004
                f"{obj!r}.__annotations__ is neither a dict nor None"
            )

        if not ann:
            return {}

        return dict(ann)



"""CQHTTP 协议适配器。

本适配器适配了 OneBot v11 协议。
协议详情请参考：[OneBot](https://github.com/howmanybots/onebot/blob/master/README.md)。
"""

import asyncio
import inspect
import json
import sys
import time
from functools import partial
from typing import (
    Any,
    Awaitable,
    Callable,
    ClassVar,
    Dict,
    Literal,
    Optional,
    Tuple,
    Type,
)

import aiohttp
from aiohttp import web

from iamai.adapter.utils import WebSocketAdapter
from iamai.log import logger
from iamai.message import BuildMessageType
from iamai.utils import PydanticEncoder

from . import event
from .config import Config
from .event import CQHTTPEvent, HeartbeatMetaEvent, LifecycleMetaEvent, MetaEvent
from .exceptions import ActionFailed, ApiNotAvailable, ApiTimeout, NetworkError
from .message import CQHTTPMessage, CQHTTPMessageSegment

__all__ = ["CQHTTPAdapter"]

EventModels = Dict[
    Tuple[Optional[str], Optional[str], Optional[str]], Type[CQHTTPEvent]
]

DEFAULT_EVENT_MODELS: EventModels = {}
for _, model in inspect.getmembers(event, inspect.isclass):
    if issubclass(model, CQHTTPEvent):
        DEFAULT_EVENT_MODELS[model.get_event_type()] = model


class CQHTTPAdapter(WebSocketAdapter[CQHTTPEvent, Config]):
    """CQHTTP 协议适配器。"""

    name = "cqhttp"
    Config = Config

    event_models: ClassVar[EventModels] = DEFAULT_EVENT_MODELS

    _api_response: Dict[str, Any]
    _api_response_cond: asyncio.Condition
    _api_id: int = 0

    def __getattr__(self, item: str) -> Callable[..., Awaitable[Any]]:
        """用于调用 API。可以直接通过访问适配器的属性访问对应名称的 API。

        Args:
            item: API 名称。

        Returns:
            用于调用 API 的函数。
        """
        return partial(self.call_api, item)

    async def startup(self) -> None:
        """初始化适配器。"""
        adapter_type = self.config.adapter_type
        if adapter_type == "ws-reverse":
            adapter_type = "reverse-ws"
        self.adapter_type = adapter_type
        self.host = self.config.host
        self.port = self.config.port
        self.url = self.config.url
        self.reconnect_interval = self.config.reconnect_interval
        self._api_response_cond = asyncio.Condition()
        await super().startup()

    async def reverse_ws_connection_hook(self) -> None:
        """反向 WebSocket 连接建立时的钩子函数。"""
        logger.info("WebSocket connected!")
        if self.config.access_token:
            assert isinstance(self.websocket, web.WebSocketResponse)
            if (
                self.websocket.headers.get("Authorization", "")
                != f"Bearer {self.config.access_token}"
            ):
                await self.websocket.close()

    async def websocket_connect(self) -> None:
        """创建正向 WebSocket 连接。"""
        assert self.session is not None
        logger.info("Tying to connect to WebSocket server...")
        async with self.session.ws_connect(
            f"ws://{self.host}:{self.port}/",
            headers=(
                {"Authorization": f"Bearer {self.config.access_token}"}
                if self.config.access_token
                else None
            ),
        ) as self.websocket:
            await self.handle_websocket()

    async def handle_websocket_msg(self, msg: aiohttp.WSMessage) -> None:
        """处理 WebSocket 消息。"""
        assert self.websocket is not None
        if msg.type == aiohttp.WSMsgType.TEXT:
            try:
                msg_dict = msg.json()
            except json.JSONDecodeError as e:
                self.bot.error_or_exception(
                    "WebSocket message parsing error, not json:", e
                )
                return

            if "post_type" in msg_dict:
                await self.handle_cqhttp_event(msg_dict)
            else:
                async with self._api_response_cond:
                    self._api_response = msg_dict
                    self._api_response_cond.notify_all()

        elif msg.type == aiohttp.WSMsgType.ERROR:
            logger.error(
                f"WebSocket connection closed "
                f"with exception {self.websocket.exception()!r}"
            )

    def _get_api_echo(self) -> int:
        self._api_id = (self._api_id + 1) % sys.maxsize
        return self._api_id

    @classmethod
    def add_event_model(cls, event_model: Type[CQHTTPEvent]) -> None:
        """添加自定义事件模型，事件模型类必须继承于 `CQHTTPEvent`。

        Args:
            event_model: 事件模型类。
        """
        cls.event_models[event_model.get_event_type()] = event_model

    @classmethod
    def get_event_model(
        cls,
        post_type: Optional[str],
        detail_type: Optional[str],
        sub_type: Optional[str],
    ) -> Type[CQHTTPEvent]:
        """根据接收到的消息类型返回对应的事件类。

        Args:
            post_type: 请求类型。
            detail_type: 事件类型。
            sub_type: 子类型。

        Returns:
            对应的事件类。
        """
        event_model = (
            cls.event_models.get((post_type, detail_type, sub_type), None)
            or cls.event_models.get((post_type, detail_type, None), None)
            or cls.event_models.get((post_type, None, None), None)
        )
        return event_model or cls.event_models[(None, None, None)]

    async def handle_cqhttp_event(self, msg: Dict[str, Any]) -> None:
        """处理 CQHTTP 事件。

        Args:
            msg: 接收到的信息。
        """
        post_type = msg.get("post_type", None)
        if post_type is None:
            event_class = self.get_event_model(None, None, None)
        else:
            event_class = self.get_event_model(
                post_type,
                msg.get(post_type + "_type", None),
                msg.get("sub_type", None),
            )

        cqhttp_event = event_class(adapter=self, **msg)

        if cqhttp_event.post_type == "meta_event":
            # meta_event 不交由插件处理
            assert isinstance(cqhttp_event, MetaEvent)
            if cqhttp_event.meta_event_type == "lifecycle":
                assert isinstance(cqhttp_event, LifecycleMetaEvent)
                if cqhttp_event.sub_type == "connect":
                    logger.info(
                        f"WebSocket connection "
                        f"from CQHTTP Bot {msg.get('self_id')} accepted!"
                    )
            elif cqhttp_event.meta_event_type == "heartbeat":
                assert isinstance(cqhttp_event, HeartbeatMetaEvent)
                if cqhttp_event.status.good and cqhttp_event.status.online:
                    pass
                else:
                    logger.error(
                        f"CQHTTP Bot status is not good: {cqhttp_event.status.model_dump()}"
                    )
        else:
            await self.handle_event(cqhttp_event)

    async def call_api(self, api: str, **params: Any) -> Any:
        """调用 CQHTTP API，协程会等待直到获得 API 响应。

        Args:
            api: API 名称。
            **params: API 参数。

        Returns:
            API 响应中的 data 字段。

        Raises:
            NetworkError: 网络错误。
            ApiNotAvailable: API 请求响应 404， API 不可用。
            ActionFailed: API 请求响应 failed， API 操作失败。
            ApiTimeout: API 请求响应超时。
        """
        assert self.websocket is not None
        api_echo = self._get_api_echo()
        try:
            await self.websocket.send_str(
                json.dumps(
                    {"action": api, "params": params, "echo": api_echo},
                    cls=PydanticEncoder,
                )
            )
        except Exception as e:
            raise NetworkError from e

        start_time = time.time()
        while not self.bot.should_exit.is_set():
            if time.time() - start_time > self.config.api_timeout:
                break
            async with self._api_response_cond:
                try:
                    await asyncio.wait_for(
                        self._api_response_cond.wait(),
                        timeout=start_time + self.config.api_timeout - time.time(),
                    )
                except asyncio.TimeoutError:
                    break
                if self._api_response["echo"] == api_echo:
                    if self._api_response.get("retcode") == ApiNotAvailable.ERROR_CODE:
                        raise ApiNotAvailable(resp=self._api_response)
                    if self._api_response.get("status") == "failed":
                        raise ActionFailed(resp=self._api_response)
                    return self._api_response.get("data")

        if not self.bot.should_exit.is_set():
            raise ApiTimeout
        return None

    async def send(
        self,
        message_: BuildMessageType[CQHTTPMessageSegment],
        message_type: Literal["private", "group"],
        id_: int,
    ) -> Any:
        """发送消息，调用 `send_private_msg` 或 `send_group_msg` API 发送消息。

        Args:
            message_: 消息内容，可以是 `str`, `Mapping`, `Iterable[Mapping]`,
                `CQHTTPMessageSegment`, `CQHTTPMessage。`
                将使用 `CQHTTPMessage` 进行封装。
            message_type: 消息类型。应该是 "private" 或者 "group"。
            id_: 发送对象的 ID， QQ 号码或者群号码。

        Returns:
            API 响应。

        Raises:
            TypeError: `message_type` 不是 "private" 或 "group"。
            ...: 同 `call_api()` 方法。
        """
        if message_type == "private":
            return await self.send_private_msg(
                user_id=id_, message=CQHTTPMessage(message_)
            )
        if message_type == "group":
            return await self.send_group_msg(
                group_id=id_, message=CQHTTPMessage(message_)
            )
        raise TypeError('message_type must be "private" or "group"')



"""iamai Exception.

The following are exceptions that may be thrown when iamai is running. Most of these exceptions do not require user handling, iamai will automatically catch and handle them.
For adapter developers, all exceptions thrown by adapters should inherit from `AdapterException`.
"""

__all__ = [
    "EventException",
    "SkipException",
    "StopException",
    "iamaiException",
    "GetEventTimeout",
    "AdapterException",
    "LoadModuleError",
]


class EventException(BaseException):
    """Exceptions thrown by plug-ins during event processing are used to control the propagation of events and will be automatically captured and processed by iamai."""


class SkipException(EventException):
    """Skip the current plugin and continue the current event propagation."""


class StopException(EventException):
    """Stop propagation of current events."""


class iamaiException(Exception):  # noqa: N818
    """Base class for all exceptions that occur with iamai."""


class GetEventTimeout(iamaiException):
    """Thrown when the get method times out."""


class AdapterException(iamaiException):
    """Base class for exceptions thrown by adapters. All exceptions thrown by adapters should inherit from this class."""


class LoadModuleError(iamaiException):
    """Loading module error, thrown when a class of a specific type cannot be found in the specified module or there are multiple qualifying classes in the module."""



"""Console适配器。"""

from asyncio import Condition

from iamai.event import Event  # noqa: F401
from iamai.adapter import Adapter

from .config import Config
from .event import ConsoleEvent

__all__ = ["ConsoleAdapter"]


class ConsoleAdapter(Adapter[ConsoleEvent, Config]):
    """Console适配器。"""

    name: str = "console"
    _msg: str = ""
    _cond: Condition
    Config = Config

    async def startup(self):
        self._cond = Condition()

    async def run(self):
        while not self.bot.should_exit.is_set():
            async with self._cond:
                await self._cond.wait()
                await self.handle_event(
                    ConsoleEvent(adapter=self, type="message", message=self._msg)
                )

    async def send(self, msg: str):
        """此方法发送的消息会直接使此适配器产生一个事件。"""
        async with self._cond:
            self._msg = msg
            self._cond.notify()



"""Configuration module.

iamai uses `pydantic <https://pydantic-docs.helpmanual.io/>`_ to read configuration.
"""

from typing import Set, Union

from pydantic import BaseModel, ConfigDict, DirectoryPath, Field

__all__ = [
    "ConfigModel",
    "LogConfig",
    "BotConfig",
    "PluginConfig",
    "AdapterConfig",
    "MainConfig",
]


class ConfigModel(BaseModel):
    """iamai configuration model.

    Attributes:
        __config_name__: Configuration name.
    """

    model_config = ConfigDict(extra="allow")

    __config_name__: str = ""


class LogConfig(ConfigModel):
    """iamai log related settings.

    Attributes:
        level: log level.
        verbose_exception: Detailed exception record. When set to ``True``, exception Traceback will be added to the log.
    """

    level: Union[str, int] = "DEBUG"
    verbose_exception: bool = False


class BotConfig(ConfigModel):
    """Bot configuration.

    Attributes:
        plugins: List of plugins to be loaded, which will be loaded by the ``load_plugins()`` method of the ``Bot`` class.
        plugin_dirs: List of plugin directories to be loaded, which will be loaded by the ``load_plugins_from_dirs()`` method of the ``Bot`` class.
        adapters: A list of adapters to be loaded, which will be loaded in turn by the ``load_adapters()`` method of the ``Bot`` class.
        log: iamai log related settings.
    """

    plugins: Set[str] = Field(default_factory=set)
    plugin_dirs: Set[DirectoryPath] = Field(default_factory=set)
    adapters: Set[str] = Field(default_factory=set)
    log: LogConfig = LogConfig()


class PluginConfig(ConfigModel):
    """Plugin configuration."""


class AdapterConfig(ConfigModel):
    """Adapter configuration."""


class MainConfig(ConfigModel):
    """iamai configuration.

    Attributes:
        bot: the main configuration of iamai.
    """

    bot: BotConfig = BotConfig()
    plugin: PluginConfig = PluginConfig()
    adapter: AdapterConfig = AdapterConfig()
